@@ -52,8 +52,12 @@
 #define KEY_DISABLE_WORKAROUNDS "/apps/metacity/general/disable_workarounds"
 #define KEY_BUTTON_LAYOUT "/apps/metacity/general/button_layout"
 #define KEY_REDUCED_RESOURCES "/apps/metacity/general/reduced_resources"
+#define KEY_GNOME_ACCESSIBILITY "/desktop/gnome/interface/accessibility"
 
 #define KEY_COMMAND_PREFIX "/apps/metacity/keybinding_commands/command_"
+
+#define KEY_TERMINAL_COMMAND "/desktop/gnome/applications/terminal/exec"
+
 #define KEY_SCREEN_BINDINGS_PREFIX "/apps/metacity/global_keybindings"
 #define KEY_WINDOW_BINDINGS_PREFIX "/apps/metacity/window_keybindings"
 
@@ -85,6 +89,7 @@ static gboolean auto_raise_delay = 500;
 static gboolean provide_visual_bell = TRUE;
 static gboolean bell_is_audible = TRUE;
 static gboolean reduced_resources = FALSE;
+static gboolean gnome_accessibility = FALSE;
 
 static MetaVisualBellType visual_bell_type = META_VISUAL_BELL_INVALID;
 static MetaButtonLayout button_layout = {
@@ -105,6 +110,8 @@ static MetaButtonLayout button_layout = {
 /* The screenshot commands are at the end */
 static char *commands[MAX_COMMANDS] = { NULL, };
 
+static char *terminal_command = NULL;
+
 static char *workspace_names[MAX_REASONABLE_WORKSPACES] = { NULL, };
 
 #ifdef HAVE_GCONF
@@ -130,9 +137,11 @@ static gboolean update_binding            (MetaKeyPref *binding,
                                            const char  *value);
 static gboolean update_command            (const char  *name,
                                            const char  *value);
+static gboolean update_terminal_command   (const char *value);
 static gboolean update_workspace_name     (const char  *name,
                                            const char  *value);
 static gboolean update_reduced_resources  (gboolean     value);
+static gboolean update_gnome_accessibility  (gboolean     value);
 
 static void change_notify (GConfClient    *client,
                            guint           cnxn_id,
@@ -304,6 +313,16 @@ meta_prefs_init (void)
                         &err);
   cleanup_error (&err);
 
+  gconf_client_add_dir (default_client, "/desktop/gnome/applications/terminal",
+                        GCONF_CLIENT_PRELOAD_RECURSIVE,
+                        &err);
+  cleanup_error (&err);
+
+  gconf_client_add_dir (default_client, KEY_GNOME_ACCESSIBILITY,
+                        GCONF_CLIENT_PRELOAD_RECURSIVE,
+                        &err);
+  cleanup_error (&err);
+
   str_val = gconf_client_get_string (default_client, KEY_MOUSE_BUTTON_MODS,
                                      &err);
   cleanup_error (&err);
@@ -395,6 +414,18 @@ meta_prefs_init (void)
 				    &err);
   cleanup_error (&err);
   update_reduced_resources (bool_val);
+
+  str_val = gconf_client_get_string (default_client, KEY_TERMINAL_COMMAND,
+                                     &err);
+  cleanup_error (&err);
+  update_terminal_command (str_val);
+  g_free (str_val);
+
+  bool_val = gconf_client_get_bool (default_client, KEY_GNOME_ACCESSIBILITY,
+                                    &err);
+  
+  cleanup_error (&err);
+  update_gnome_accessibility (bool_val);
 #endif /* HAVE_GCONF */
   
   /* Load keybindings prefs */
@@ -412,6 +443,17 @@ meta_prefs_init (void)
                            NULL,
                            NULL,
                            &err);
+  gconf_client_notify_add (default_client, KEY_TERMINAL_COMMAND,
+                           change_notify,
+                           NULL,
+                           NULL,
+                           &err);
+  gconf_client_notify_add (default_client, KEY_GNOME_ACCESSIBILITY,
+                           change_notify,
+                           NULL,
+                           NULL,
+                           &err);
+
   cleanup_error (&err);  
 #endif /* HAVE_GCONF */
 }
@@ -679,6 +721,22 @@ change_notify (GConfClient    *client,
       if (update_command (key, str))
         queue_changed (META_PREF_COMMANDS);
     }
+  else if (strcmp (key, KEY_TERMINAL_COMMAND) == 0)
+    {
+      const char *str;
+
+      if (value && value->type != GCONF_VALUE_STRING)
+        {
+          meta_warning (_("GConf key \"%s\" is set to an invalid type\n"),
+                        KEY_TERMINAL_COMMAND);
+          goto out;
+        }
+      
+      str = value ? gconf_value_get_string (value) : NULL;
+
+      if (update_terminal_command (str))
+        queue_changed (META_PREF_TERMINAL_COMMAND);
+    }
   else if (str_has_prefix (key, KEY_WORKSPACE_NAME_PREFIX))
     {
       const char *str;
@@ -772,6 +830,22 @@ change_notify (GConfClient    *client,
       if (update_reduced_resources (b))
         queue_changed (META_PREF_REDUCED_RESOURCES);
     }
+  else if (strcmp (key, KEY_GNOME_ACCESSIBILITY) == 0)
+    {
+      gboolean b;
+
+      if (value && value->type != GCONF_VALUE_BOOL)
+        {
+          meta_warning (_("GConf key \"%s\" is set to an invalid type\n"),
+                       KEY_GNOME_ACCESSIBILITY);
+          goto out;
+        }
+
+      b = value ? gconf_value_get_bool (value) : gnome_accessibility;
+
+      if (update_gnome_accessibility (b))
+        queue_changed (META_PREF_GNOME_ACCESSIBILITY);
+    }
   else
     {
       meta_topic (META_DEBUG_PREFS, "Key %s doesn't mean anything to Metacity\n",
@@ -1288,6 +1362,16 @@ update_reduced_resources (gboolean value)
 
   return old != reduced_resources;
 }
+
+static gboolean
+update_gnome_accessibility (gboolean value)
+{
+  gboolean old = gnome_accessibility;
+
+  gnome_accessibility = value;
+  
+  return old != gnome_accessibility;
+}
 #endif /* HAVE_GCONF */
 
 #ifdef WITH_VERBOSE_MODE
@@ -1335,6 +1419,9 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_COMMANDS:
       return "COMMANDS";
 
+    case META_PREF_TERMINAL_COMMAND:
+      return "TERMINAL_COMMAND";
+
     case META_PREF_BUTTON_LAYOUT:
       return "BUTTON_LAYOUT";
       break;
@@ -1358,6 +1445,9 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_REDUCED_RESOURCES:
       return "REDUCED_RESOURCES";
       break;
+    case META_PREF_GNOME_ACCESSIBILITY:
+      return "GNOME_ACCESSIBILTY";
+      break;
     }
 
   return "(unknown)";
@@ -1457,6 +1547,7 @@ static MetaKeyPref screen_bindings[] = {
   { META_KEYBINDING_COMMAND_32, 0, 0, FALSE },
   { META_KEYBINDING_COMMAND_SCREENSHOT, 0, 0, FALSE },
   { META_KEYBINDING_COMMAND_WIN_SCREENSHOT, 0, 0, FALSE },
+  { META_KEYBINDING_RUN_COMMAND_TERMINAL, 0, 0, FALSE },
   { NULL, 0, 0, FALSE}
 };
 
@@ -1776,6 +1867,31 @@ update_command (const char  *name,
   
   return TRUE;
 }
+
+static gboolean
+update_terminal_command (const char *value)
+{
+  char *old_terminal_command;
+  gboolean changed;
+  
+  old_terminal_command = terminal_command;
+  
+  if (value != NULL && *value)
+    {
+      terminal_command = g_strdup (value);
+    }
+
+  changed = TRUE;
+  if ((old_terminal_command && terminal_command &&
+       strcmp (old_terminal_command, terminal_command) == 0) ||
+      (old_terminal_command == NULL && terminal_command == NULL))
+    changed = FALSE;
+
+  if (old_terminal_command != terminal_command)
+    g_free (old_terminal_command);
+
+  return changed;
+}
 #endif /* HAVE_GCONF */
 
 const char*
@@ -1807,6 +1923,18 @@ meta_prefs_get_gconf_key_for_command (int i)
   return key;
 }
 
+const char*
+meta_prefs_get_terminal_command (void)
+{
+  return terminal_command;
+}
+
+const char*
+meta_prefs_get_gconf_key_for_terminal_command (void)
+{
+  return KEY_TERMINAL_COMMAND;
+}
+
 #ifdef HAVE_GCONF
 static gboolean
 update_workspace_name (const char  *name,
@@ -1975,19 +2103,19 @@ meta_prefs_get_button_layout (MetaButtonLayout *button_layout_p)
 }
 
 gboolean
-meta_prefs_get_visual_bell ()
+meta_prefs_get_visual_bell (void)
 {
   return provide_visual_bell;
 }
 
 gboolean
-meta_prefs_bell_is_audible ()
+meta_prefs_bell_is_audible (void)
 {
   return bell_is_audible;
 }
 
 MetaVisualBellType
-meta_prefs_get_visual_bell_type ()
+meta_prefs_get_visual_bell_type (void)
 {
   return visual_bell_type;
 }
@@ -2010,29 +2138,35 @@ meta_prefs_get_window_bindings (const MetaKeyPref **bindings,
 }
 
 MetaActionDoubleClickTitlebar
-meta_prefs_get_action_double_click_titlebar ()
+meta_prefs_get_action_double_click_titlebar (void)
 {
   return action_double_click_titlebar;
 }
 
 gboolean
-meta_prefs_get_auto_raise ()
+meta_prefs_get_auto_raise (void)
 {
   return auto_raise;
 }
 
 int
-meta_prefs_get_auto_raise_delay ()
+meta_prefs_get_auto_raise_delay (void)
 {
   return auto_raise_delay;
 }
 
 gboolean
-meta_prefs_get_reduced_resources ()
+meta_prefs_get_reduced_resources (void)
 {
   return reduced_resources;
 }
 
+gboolean
+meta_prefs_get_gnome_accessibility ()
+{
+  return gnome_accessibility;
+}
+
 MetaKeyBindingAction
 meta_prefs_get_keybinding_action (const char *name)
 {
